@@ -170,10 +170,12 @@ INSTANTIATE_TEST_SUITE_P(
         {"hlsl.basic.geom", "main"},
         {"hlsl.boolConv.vert", "main"},
         {"hlsl.buffer.frag", "PixelShaderFunction"},
+        {"hlsl.buffer-offsets.comp", "main"},
         {"hlsl.calculatelod.dx10.frag", "main"},
         {"hlsl.calculatelodunclamped.dx10.frag", "main"},
         {"hlsl.cast.frag", "PixelShaderFunction"},
         {"hlsl.cbuffer-identifier.vert", "main"},
+        {"hlsl.cbuffer-offsets.comp", "main"},
         {"hlsl.charLit.vert", "main"},
         {"hlsl.clip.frag", "main"},
         {"hlsl.clipdistance-1.frag", "main"},
@@ -419,6 +421,7 @@ INSTANTIATE_TEST_SUITE_P(
         {"hlsl.structIoFourWay.frag", "main"},
         {"hlsl.structStructName.frag", "main"},
         {"hlsl.subpass.frag", "main"},
+        {"hlsl.swizzle.vec1.comp", "main"},
         {"hlsl.synthesizeInput.frag", "main"},
         {"hlsl.texturebuffer.frag", "main"},
         {"hlsl.texture.struct.frag", "main"},
